@@ -1,17 +1,19 @@
 from __future__ import annotations
 
 import os
+import sqlite3
 import sys
+import threading
 from functools import lru_cache
 from pathlib import Path
 from typing import TYPE_CHECKING, Any, cast
 
 import hishel
+import hishel.httpx
 import httpx
 from unearth.fetchers import PyPIClient
 
 from pdm.__version__ import __version__
-from pdm.models.serializers import MsgPackSerializer
 from pdm.termui import logger
 
 if TYPE_CHECKING:
@@ -55,6 +57,52 @@ def _get_transport(
     return httpx.HTTPTransport(verify=verify, cert=cert, trust_env=True, proxy=proxy, retries=MAX_RETRIES)
 
 
+class ThreadedSyncSqliteStorage(hishel.SyncSqliteStorage):
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
+        self._local_conns: dict[int, sqlite3.Connection] = {}
+        self._lock = threading.Lock()
+        self._initialized = False
+        super().__init__(*args, **kwargs)
+
+    @property
+    def connection(self) -> sqlite3.Connection | None:
+        return self._local_conns.get(threading.get_ident())
+
+    @connection.setter
+    def connection(self, conn: sqlite3.Connection | None) -> None:
+        if conn is not None:
+            self._local_conns[threading.get_ident()] = conn
+
+    def close(self) -> None:
+        with self._lock:
+            while self._local_conns:
+                _, conn = self._local_conns.popitem()
+                conn.close()
+
+    def _ensure_connection(self) -> sqlite3.Connection:
+        """
+        Ensure connection is established and database is initialized.
+
+        We need to create connections with check_same_thread=False for the sake of close(),
+        so we have to open-code the entire implementation here.
+        """
+
+        # we take a lock _despite_ using TLS to protect against concurrent close(), otherwise we have a TOCTOU
+        # this is kinda ugly and defeats the purpose of using TLS, but eh
+        with self._lock:
+            if self.connection is None:
+                # Create cache directory and resolve full path on first connection
+                self.database_path.parent.mkdir(parents=True, exist_ok=True)
+                full_path = self.database_path.resolve()
+                conn = sqlite3.connect(str(full_path), check_same_thread=False)
+                conn.execute("PRAGMA journal_mode=WAL")
+                self.connection = conn
+            if not self._initialized:
+                self._initialize_database()
+                self._initialized = True
+            return self.connection
+
+
 class PDMPyPIClient(PyPIClient):
     def __init__(self, *, sources: list[RepositoryConfig], cache_dir: Path | None = None, **kwargs: Any) -> None:
         from httpx._utils import URLPattern
@@ -65,11 +113,16 @@ class PDMPyPIClient(PyPIClient):
             def cache_transport(transport: httpx.BaseTransport) -> httpx.BaseTransport:
                 return transport
         else:
-            storage = hishel.FileStorage(serializer=MsgPackSerializer(), base_path=cache_dir, ttl=CACHES_TTL)
-            controller = hishel.Controller()
+            # clean up old (pre-hishel 1.0) cache
+            cache_db = cache_dir / "http-cache.db"
+            if not cache_db.exists():
+                for f in cache_dir.iterdir():
+                    if not f.name.startswith("http-cache.db"):
+                        f.unlink()
+            storage = ThreadedSyncSqliteStorage(database_path=cache_db, default_ttl=CACHES_TTL)
 
             def cache_transport(transport: httpx.BaseTransport) -> httpx.BaseTransport:
-                return hishel.CacheTransport(transport, storage, controller)
+                return hishel.httpx.SyncCacheTransport(next_transport=transport, storage=storage)
 
         mounts: dict[str, httpx.BaseTransport] = {"file://": LocalFSTransport()}
         self._trusted_host_ports: set[tuple[str, int | None]] = set()
