@@ -5,7 +5,7 @@ import pytest
 
 from pdm.models.backends import _BACKENDS, get_backend, get_relative_path
 from pdm.project import Project
-from pdm.utils import cd, path_to_url
+from pdm.utils import cd
 from tests import FIXTURES
 
 
@@ -27,18 +27,18 @@ def _setup_backend(project: Project, backend: str):
 
 
 @pytest.mark.parametrize("backend", _BACKENDS.keys())
-def test_project_backend(project, working_set, backend, invoke):
+def test_project_backend(project, working_set, backend, pdm):
     _setup_backend(project, backend)
     shutil.copytree(FIXTURES / "projects/demo", project.root / "demo")
     project.root.joinpath("sub").mkdir()
     with cd(project.root.joinpath("sub")):
-        invoke(["add", "--no-self", "../demo"], obj=project, strict=True)
+        pdm(["add", "--no-self", "../demo"], obj=project, strict=True)
         assert "idna" in working_set
         assert "demo" in working_set
         dep = project.pyproject.metadata["dependencies"][0]
-        demo_path = project.root.joinpath("demo").as_posix()
-        demo_url = path_to_url(demo_path)
-        if backend in ("pdm-pep517", "pdm-backend"):
+        demo_path = project.root.joinpath("demo")
+        demo_url = demo_path.as_uri()
+        if backend == "pdm-backend":
             assert dep == "demo @ file:///${PROJECT_ROOT}/demo"
         elif backend == "hatchling":
             assert dep == "demo @ {root:uri}/demo"
@@ -54,7 +54,7 @@ def test_project_backend(project, working_set, backend, invoke):
 
 def test_hatch_expand_variables(monkeypatch):
     root = Path().absolute()
-    root_url = path_to_url(root.as_posix())
+    root_url = root.as_uri()
     backend = get_backend("hatchling")(root)
     monkeypatch.setenv("BAR", "bar")
     assert backend.expand_line("demo @ {root:uri}/demo") == f"demo @ {root_url}/demo"
@@ -63,10 +63,10 @@ def test_hatch_expand_variables(monkeypatch):
     assert backend.relative_path_to_url("../demo") == "{root:uri}/../demo"
 
 
-def test_pdm_pep517_expand_variables(monkeypatch):
+def test_pdm_backend_expand_variables(monkeypatch):
     root = Path().absolute()
-    root_url = path_to_url(root.as_posix())
-    backend = get_backend("pdm-pep517")(root)
+    root_url = root.as_uri()
+    backend = get_backend("pdm-backend")(root)
     monkeypatch.setenv("BAR", "bar")
     assert backend.expand_line("demo @ file:///${PROJECT_ROOT}/demo") == f"demo @ {root_url}/demo"
     assert backend.expand_line("demo==${BAR}") == "demo==bar"
